unit main;

{$ifdef fpc}
  {$mode objfpc}{$H+}
{$endif}

interface

uses
  Classes,
  SysUtils,
  Forms,
  Controls,
  Graphics,
  Dialogs,
  StdCtrls,
  deltime,
  wolfutil;

type

  { TMainForm }

  TMainForm = class(TForm)
    Memo    : TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
  private
    Start   : Boolean;
  end;

var
  MainForm : TMainForm;

implementation

{$ifdef fpc}
  {$R *.lfm}
{$else}
  {$R *.dfm}
{$endif}

{ TMainForm }

procedure TMainForm.FormPaint(Sender: TObject);
  var     l    : LongestInt;
          i    : LongInt;
          s, u : String;
          TZif : Array[0 .. sizeof(tTZif)] of char;
          t    : tDateTime;
  begin
    if Start then
      begin
        Start := False;
        if tzhead <> Nil then
          begin
            strlcopy(TZif, tzhead^.tzh_TZif, sizeof(tTZif));
            Memo.Lines.add('tzhead^.tzh_TZif       = "' + TZif + '"');
            s := '';
            for i := 0 to sizeof(tzhead^.tzh_reserved) - 1 do
              s := s + '[' + IntToHex(tzhead^.tzh_reserved[i], 2) + ']';
            Memo.Lines.add('tzhead^.tzh_reserved   = ' + s);
            Memo.Lines.add('tzhead^.tzh_ttisgmtcnt = '
                           + IntToStr(tzhead^.tzh_ttisgmtcnt));
            Memo.Lines.add('tzhead^.tzh_ttisstdcnt = '
                           + IntToStr(tzhead^.tzh_ttisstdcnt));
            Memo.Lines.add('tzhead^.tzh_leapcnt    = '
                           + IntToStr(tzhead^.tzh_leapcnt));
            Memo.Lines.add('tzhead^.tzh_timecnt    = '
                           + IntToStr(tzhead^.tzh_timecnt));
            Memo.Lines.add('tzhead^.tzh_typecnt    = '
                           + IntToStr(tzhead^.tzh_typecnt));
            Memo.Lines.add('tzhead^.tzh_charcnt    = '
                           + IntToStr(tzhead^.tzh_charcnt));
            for i := 0 to tzhead^.tzh_timecnt - 1 do
              begin
                s := '';
                if transitions <> Nil then
                  begin
                    {$R-}
                      t := EncodeDate(1970, 1, 1)
                           + transitions^[i] / 60 / 60 / 24;
                    {$R+}
                      s := 'transitions^[' + IntToDez(i, 3, False) + '] = '
                           + FormatDateTime(GetLongDateFormat
                                            + ' ' + GetLongTimeFormat, t)
                           + ' (GMT)';
                  end;
                if ttinfoind <> Nil then
                  begin
                    {$R-}
                      if s > '' then
                        s := s + ', ';
                      s := s + 'ttinfoind^[' + IntToDez(i, 3, False) + '] = '
                             + IntToStr(ttinfoind^[i]);
                    {$R+}
                  end;
                if s >'' then
                  Memo.Lines.add(s);
              end;
            if ttinfo <> Nil then
              for i := 0 to tzhead^.tzh_typecnt - 1 do
                begin
                  {$R-}
                    s := IntToDez(ttinfo^[i].tt_gmtoff, 1, True);
                    s := 'tt_gmtoff: ' + StringOfChar(' ', 6 - length(s)) + s
                         + ', tt_isdst: ';
                    if ttinfo^[i].tt_isdst then
                      s := s + 'True '
                    else
                      s := s + 'False';
                    s := s + ', tt_abbrind: '
                           + IntToStr(ttinfo^[i].tt_abbrind);
                  {$R+}
                  Memo.Lines.add('ttinfo^[' + IntToStr(i) + '] = ' + s);
                end;
            if tzNames <> Nil then
              for i := 0 to tzNames.Count - 1 do
                Memo.Lines.add('tznames[' + IntToDez(i, 2, False) + '] = "'
                               + tznames[i] + '"');
            if leap <> Nil then
              for i := 0 to tzhead^.tzh_leapcnt - 1 do
                begin
                  {$R-}
                    s := FormatDateTime(GetLongDateFormat
                                        + ' ' + GetLongTimeFormat,
                                        EncodeDate(1970, 1, 1)
                                        + leap^[i].transition / 60 / 60 / 24)
                         + ' (GMT)';
                    Memo.Lines.add('leap^[' + IntToStr(i) + '] = on ' + s
                                   + ' ' + IntToStr(leap^[i].change)
                                   + ' seconds');
                  {$R+}
                end;
            if std_wall <> nil then
              for i := 0 to tzhead^.tzh_ttisstdcnt - 1 do
                {$R-}
                  if std_wall^[i] then
                    Memo.Lines.add('std_wall^[' + IntToStr(i) + '] = True')
                  else
                    Memo.Lines.add('std_wall^[' + IntToStr(i) + '] = False');
                {$R+}
            if utc_local <> nil then
              for i := 0 to tzhead^.tzh_ttisgmtcnt - 1 do
                {$R-}
                  if utc_local^[i] then
                    Memo.Lines.add('utc_local^[' + IntToStr(i) + '] = True')
                  else
                    Memo.Lines.add('utc_local^[' + IntToStr(i) + '] = False');
                {$R+}
            if tzMoreData <> Nil then
              begin
                s := '';
                l := 0;
                while l < tzMoreLen do
                  begin
                    {$R-}
                      if (ord(' ') <= tzMoreData^[l]) and (tzMoreData^[l] < $7f)
                      then
                        s := s + chr(tzMoreData^[l])
                      else
                        s := s + '[' + IntToHex(tzMoreData^[l], 2) + ']';
                    {$R+}
                    inc(l);
                    if Canvas.TextWidth(s + '########') >= Memo.ClientWidth then
                      begin
                        Memo.Lines.add(s);
                        s := '';
                      end;
                  end;
                Memo.Lines.add(s);
              end;
            Memo.Lines.add('');
            if transitions <> Nil then
              for i := 0 to tzhead^.tzh_timecnt - 1 do
                begin
                  s := 'GMT';
                  l := 0;
                  {$R-}
                    t := EncodeDate(1970, 1, 1)
                         + transitions^[i] / 60 / 60 / 24;
                  {$R+}
                  if ttinfoind <> Nil then
                    if ttinfo <> Nil then
                      {$R-}
                        if ttinfoind^[i] < tzhead^.tzh_typecnt then
                          begin
                            s := '';
                            if i - 1 >= 0 then
                              begin
                                if ttinfoind^[i - 1] < tzhead^.tzh_typecnt
                                then
                                  l := ttinfo^[ttinfoind^[i - 1]].
                                         tt_gmtoff div 60;
                              end
                            else
                              if i + 1 < tzhead^.tzh_timecnt then
                                begin
                                  if ttinfoind^[i + 1] < tzhead^.tzh_typecnt
                                  then
                                    l := ttinfo^[ttinfoind^[i + 1]].
                                           tt_gmtoff div 60;
                                end;
                            t := t + l / 60 / 24;
                            l := ttinfo^[ttinfoind^[i]].tt_gmtoff div 60;
                            if tznames <> Nil then
                              if ttinfo^[ttinfoind^[i]].tt_abbrind
                                 < tznames.count
                              then
                                s := tznames[ttinfo^[ttinfoind^[i]].tt_abbrind];
                          end;
                      {$R+}
                s := s + StringOfChar(' ', 4 - length(s));
                u := IntToStr(-l);
                u := StringOfChar(' ', 4 - length(u)) + u;
                Memo.Lines.add('transition ' + IntToDez(i, 3, False) + ': '
                               + FormatDateTime(GetLongDateFormat + ' '
                                                + GetLongTimeFormat, t)
                               + ' ' + s + ', Bias ' + u);
              end;
          end;
      end;
  end;

procedure TMainForm.FormCreate(Sender: TObject);
  begin
    Memo.align := alClient;
    Start      := True;
  end;

end.

